## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005

from CvPythonExtensions import *
import CvUtil
import PyHelpers
import Popup as PyPopup

# globals
gc = CyGlobalContext()
PyPlayer = PyHelpers.PyPlayer

class CustomFunctions:

    def __init__(self):
		
            self.Distance = [[0] * 11 for i in range(11)]
            for iX in range(11):
                for iY in range(11):
                    if iX in [0, 1] and iY in [0, 1]:
                        self.Distance[iX][iY] = 1
                    else:
                        self.Distance[iX][iY] = int((iX**2 + iY**2)**0.5 + 0.75)

            self.CultureDistance = []
            for i in range(16):
                self.CultureDistance.append(0)
            for i in range(16):
                if i in [0, 1]:
                    self.CultureDistance[i] = 1
                else:
                    self.CultureDistance[i] = (0.75)**(i-1)

    def ChangeOldPlotCulture(self, iPlayer, pCity, Amount):

            if Amount == 0: #If this is true, none of this matters.
                return
#           This function mimics the way plot culture is assigned.  Works for a radius of up to 10.
            CityX = pCity.getX()
            CityY = pCity.getY()
            TargetCultureLevel = 0
            CurrentCulture = pCity.getCulture(iPlayer)
#           First, determine what the current culture level is for the player this culture is being assigned to.
            for i in range(gc.getNumCultureLevelInfos()):
                if CurrentCulture >= 50 * 5**TargetCultureLevel:
                    TargetCultureLevel += 1
            TargetCultureLevel += 1
#           Cycle through every plot within a 21x21 square centered on the city and assign plot culture.
            for iiX in range(21):
                for iiY in range(21):
                    iX = iiX-10 + CityX
                    iY = iiY-10 + CityY
                    FromX = abs(iiX - 10)
                    FromY = abs(iiY - 10)
                    if CyMap().isPlot(iX, iY):
                        if TargetCultureLevel - self.Distance[FromX][FromY] >= 0:
                            if Amount > 0:
                                CultureChange = ((TargetCultureLevel - self.Distance[FromX][FromY]) * 20 + Amount + 1)
                                CyMap().plot(iX, iY).changeCulture(iPlayer, CultureChange)
                            else:
                                CultureChange = ((TargetCultureLevel - self.Distance[FromX][FromY]) * 20 + -Amount + 1)
                                CyMap().plot(iX, iY).changeCulture(iPlayer, -CultureChange)

    def ChangeNewPlotCulture(self, iPlayer, pCity, Amount, BonusOne):

            if Amount == 0: #If this is true, none of this matters.
                return
#           This function assigns culture in a new way.  Works for a radius of up to 10.
            CityX = pCity.getX()
            CityY = pCity.getY()
            TargetCultureLevel = 0
            CurrentCulture = pCity.getCulture(iPlayer)
#           First, determine what the current culture level is for the player this culture is being assigned to.
            for i in range(gc.getNumCultureLevelInfos()):
                if CurrentCulture >= 50 * 5**TargetCultureLevel:
                    TargetCultureLevel += 1
            TargetCultureLevel += 1
#           Cycle through every plot within a 21x21 square centered on the city and assign plot culture.
            for iiX in range(21):
                for iiY in range(21):
                    iX = iiX-10 + CityX
                    iY = iiY-10 + CityY
                    FromX = abs(iiX - 10)
                    FromY = abs(iiY - 10)
                    if CyMap().isPlot(iX, iY):
                        if TargetCultureLevel - self.Distance[FromX][FromY] >= 0:
                            if Amount > 0:
                                CultureChange = int(Amount * self.CultureDistance[self.Distance[FromX][FromY]])
                                if BonusOne == True:
                                    CultureChange += 1
                                CyMap().plot(iX, iY).changeCulture(iPlayer, CultureChange)
                            else:
                                CultureChange = int(-Amount * self.CultureDistance[self.Distance[FromX][FromY]])
                                if BonusOne == True:
                                    CultureChange += 1
                                CyMap().plot(iX, iY).changeCulture(iPlayer, -CultureChange)

    def Inquisit(self, pCity):
        
            szReligionList = [ ]
            StateBelief = gc.getPlayer(pCity.getOwner()).getStateReligion()
            for i in range(gc.getNumReligionInfos()):
                if (not StateBelief == i and pCity.isHasReligion(i) and not pCity.isHolyCityByType(i)):
                    szReligionList.append(i)
            if len(szReligionList) != 0:
                iTarget = szReligionList[CyRandom().get(len(szReligionList), "Get a random number for religion removal.")]
                pCity.setHasReligion(iTarget, False, True, True)
                for i in range(gc.getNumBuildingInfos()):
                    if gc.getBuildingInfo(i).getPrereqReligion() == iTarget:
                        pCity.setHasRealBuilding(i, False)
            CyInterface().addMessage(CyGame().getActivePlayer(),True,25,'The Inquisition strikes again!','AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/Actions/Pillage.dds',ColorTypes(7),pCity.getX(),pCity.getY(),True,True)

    def CalculateFoodBonus(self, pCity):
#               Check this city for bonuses 9-19 and 28 - 30 and appropriate buildings and determine food bonus
                FoodBonus = 0
                FoodBonuses = [ gc.getInfoTypeForString('BONUS_BANANA'), gc.getInfoTypeForString('BONUS_CLAM'), gc.getInfoTypeForString('BONUS_CORN'), gc.getInfoTypeForString('BONUS_COW'), gc.getInfoTypeForString('BONUS_CRAB'), gc.getInfoTypeForString('BONUS_DEER'), gc.getInfoTypeForString('BONUS_FISH'), gc.getInfoTypeForString('BONUS_PIG'), gc.getInfoTypeForString('BONUS_RICE'), gc.getInfoTypeForString('BONUS_SHEEP'), gc.getInfoTypeForString('BONUS_WHEAT') ]
                GranaryBonuses = [ gc.getInfoTypeForString('BONUS_CORN'), gc.getInfoTypeForString('BONUS_RICE'), gc.getInfoTypeForString('BONUS_WHEAT') ]
                GrocerBonuses = [ gc.getInfoTypeForString('BONUS_BANANA'), gc.getInfoTypeForString('BONUS_SUGAR'), gc.getInfoTypeForString('BONUS_SPICES'), gc.getInfoTypeForString('BONUS_WINE'), gc.getInfoTypeForString('BONUS_SALT') ]
                SupermarketBonuses = [ gc.getInfoTypeForString('BONUS_COW'), gc.getInfoTypeForString('BONUS_DEER'), gc.getInfoTypeForString('BONUS_PIG'), gc.getInfoTypeForString('BONUS_SHEEP') ]
                HarborBonuses = [ gc.getInfoTypeForString('BONUS_CLAM'), gc.getInfoTypeForString('BONUS_CRAB'), gc.getInfoTypeForString('BONUS_FISH') ]
#                FoodBonusBuildings = [ gc.getInfoTypeForString('BUILDING_GRANARY'), gc.getInfoTypeForString('BUILDING_GROCER'), gc.getInfoTypeForString('BUILDING_SUPERMARKET'), gc.getInfoTypeForString('BUILDING_GRANARY')
#                FoodBonuses = [ BonusTypes.BONUS_BANANA, BonusTypes.BONUS_CLAM, BonusTypes.BONUS_CORN, BonusTypes.BONUS_COW, BonusTypes.BONUS_CRAB, BonusTypes.BONUS_DEER, BonusTypes.BONUS_FISH, BonusTypes.BONUS_PIG, BonusTypes.BONUS_RICE, BonusTypes.BONUS_SHEEP, BonusTypes.BONUS_WHEAT ]
#                GranaryBonuses = [ BonusTypes.BONUS_CORN, BonusTypes.BONUS_RICE, BonusTypes.BONUS_WHEAT ]
#                GrocerBonuses = [ BonusTypes.BONUS_BANANA, BonusTypes.BONUS_SUGAR, BonusTypes.BONUS_SPICES, BonusTypes.BONUS_WINE ]
 #               SupermarketBonuses = [ BonusTypes.BONUS_COW, BonusTypes.BONUS_DEER, BonusTypes.BONUS_PIG, BonusTypes.BONUS_SHEEP ]
#                HarborBonuses = [ BonusTypes.BONUS_CLAM, BonusTypes.BONUS_CRAB, BonusTypes.BONUS_FISH ]
                if (not pCity.isDisorder()):
                    for i in range(gc.getNumBonusInfos()):
                        if pCity.hasBonus(i):
                            if i in FoodBonuses:
                                FoodBonus += 0.05
                            if pCity.hasBuilding(gc.getInfoTypeForString('BUILDING_GRANARY')):
                                if i in GranaryBonuses:
                                    FoodBonus += 0.05
                            if pCity.hasBuilding(gc.getInfoTypeForString('BUILDING_GROCER')):
                                if i in GrocerBonuses:
                                    FoodBonus += 0.05
                            if pCity.hasBuilding(gc.getInfoTypeForString('BUILDING_SUPERMARKET')):
                                if i in SupermarketBonuses:
                                    FoodBonus += 0.05
                                if i == gc.getInfoTypeForString('BONUS_SALT'):
                                    FoodBonus -= 0.05
                            if pCity.hasBuilding(gc.getInfoTypeForString('BUILDING_HARBOR')):
                                if i in HarborBonuses:
                                    FoodBonus += 0.05
                    FoodBonus += 0.05 * pCity.getFreeSpecialistCount(gc.getInfoTypeForString('SPECIALIST_HYDROPONICS_FARM'))
                    FoodBonus = int(pCity.getBaseYieldRate(YieldTypes.YIELD_FOOD) * FoodBonus)
                    return FoodBonus
#                    if not pCity.isFoodProduction():
#                        pCity.changeFood(FoodBonus)
#                    else:
#                        pCity.changeProduction(FoodBonus)

    def GiveProductionBonus(self, pCity):
#               Check this city for bonuses 0 to 8 (Alunimum, horses, coal, oil, ect)
                ProductionBonus = 0
                StrategicBonuses = [ gc.getInfoTypeForString('BONUS_ALUMINUM'), gc.getInfoTypeForString('BONUS_COAL'), gc.getInfoTypeForString('BONUS_COPPER'), gc.getInfoTypeForString('BONUS_HORSE'), gc.getInfoTypeForString('BONUS_IRON'), gc.getInfoTypeForString('BONUS_MARBLE'), gc.getInfoTypeForString('BONUS_OIL'), gc.getInfoTypeForString('BONUS_STONE'), gc.getInfoTypeForString('BONUS_URANIUM') ]
                if (not pCity.isDisorder()):
                    for i in range(gc.getNumBonusInfos()):
                        if i in StrategicBonuses and pCity.hasBonus(i):
                            ProductionBonus += 0.05
                    ProductionBonus += 0.05 * pCity.getFreeSpecialistCount(gc.getInfoTypeForString('SPECIALIST_AUTOMATED_ASSEMBLY_PLANT'))
                    ProductionBonus = int(pCity.getBaseYieldRate(YieldTypes.YIELD_PRODUCTION) * ProductionBonus)
                    pCity.changeProduction(ProductionBonus)

    def AssignCustomCulture(self, pCity, iOwner):

#               Get number of players for next 2 functions              
                NumPlayers = 0
		for i in range(gc.getMAX_CIV_PLAYERS()):
			if (gc.getPlayer(i).isAlive()):
				loopPlayer = PyPlayer(gc.getPlayer(i).getID())
				if (loopPlayer.isNone() or not loopPlayer.isAlive()):
					continue
				else:
					NumPlayers += 1

#               Perform Culture Assimilation - Disabled, 1% was far too high
#                for i in range(NumPlayers):
#                    if not i == iOwner:
#                        CultureAssimilation = int(-pCity.getCulture(i) * 0.01)
#                        pCity.changeCulture(i, CultureAssimilation, False)

#               Initialize Values
                CultureModifier = pCity.getCommerceRateModifier(gc.getInfoTypeForString('COMMERCE_CULTURE')) / 100.0 + 1
                OldCultureRate = pCity.getCommerceRate(gc.getInfoTypeForString('COMMERCE_CULTURE'))
                NewCultureRate = int(pCity.getPopulation() * 2 * CultureModifier + OldCultureRate)
                CultureChange = []
                for i in range(NumPlayers):
                    CultureChange.append(0)

#               Assign culture based on nationality
                for i in range(NumPlayers):
                    if not i == iOwner:
                        CultureNationality = int( ( 0.6 + 0.4 * (1 - pCity.plot().calculateCulturePercent(i) / 100.0) ) * pCity.plot().calculateCulturePercent(i) / 100.0 * NewCultureRate )
                        CultureChange[i] += CultureNationality
                    else:
                        CultureNationality = int( (0.4 + 0.6 * pCity.plot().calculateCulturePercent(i) / 100.0) * NewCultureRate + 1)
                        CultureChange[i] += CultureNationality

#		Assign culture based on trade routes
#               Culture is assigned based on the culture rate of the trade city, the nationality of the trade city, and the value of the trade city.
#               A mixed nationality trade city will spread its mixed nationality.
#               A culturally strong trade city will spread more culture per commerce than a poor one.
#               A economically strong trade city will spread more culture than an economically poor one.
                routes = pCity.getTradeRoutes()
                for i in range(routes):
                    if (pCity.getTradeCity(i).getName()):
                        tradecity = pCity.getTradeCity(i)
                        Nation = tradecity.getOwner()
                        iTradeValue = pCity.calculateTradeProfit(tradecity)
                        TradeCityCultureRate = (tradecity.getCommerceRateModifier(gc.getInfoTypeForString('COMMERCE_CULTURE')) / 100.0 + 1) * tradecity.getPopulation() * 2 + tradecity.getCommerceRate(gc.getInfoTypeForString('COMMERCE_CULTURE'))
			TradeCityCulture = NewCultureRate * iTradeValue * 2 / 100.0
                        for n in range(NumPlayers):
                            if not n == iOwner:
                                if not n == Nation:
                                    TradeCultureNationality = int( (0.6 + 0.4 * (1 - pCity.plot().calculateCulturePercent(n) / 100.0) ) * tradecity.plot().calculateCulturePercent(n) / 100.0 * TradeCityCulture)
                                    self.ChangeNewPlotCulture(n, tradecity, TradeCultureNationality, False)
                                else:
                                    TradeCultureNationality = int( (0.4 + 0.6 * pCity.plot().calculateCulturePercent(i) / 100.0) * TradeCityCulture)
                                    self.ChangeNewPlotCulture(n, tradecity, TradeCultureNationality, False)
                            else:
                                TradeCultureNationality = int( (0.4 + 0.6 * pCity.plot().calculateCulturePercent(i) / 100.0) * TradeCityCulture / 2.0)
                                self.ChangeNewPlotCulture(n, tradecity, TradeCultureNationality, False)

#		Assign culture to the holy city owners' nations based on religions present.
		for i in range(gc.getNumReligionInfos()):
                    if pCity.isHasReligion(i):
			HolyCityCity = CyGame().getHolyCity(i)
			Nation = HolyCityCity.getOwner()
                        CultureAmount = 1
                        for n in range(gc.getNumBuildingInfos()):
                            if pCity.hasBuilding(n):
                                if gc.getBuildingInfo(n).getPrereqReligion() == i:
                                    CultureAmount += 1
                        CultureAmount = int(CultureAmount * CultureModifier)
                        CultureChange[Nation] += CultureAmount
                    
#               Remove the default culture rate
                pCity.changeCulture(iOwner, -OldCultureRate, False)
                self.ChangeOldPlotCulture(iOwner, pCity, -OldCultureRate)
#               Reassign culture based on the new model
                for i in range(NumPlayers):
                    pCity.changeCulture(i, CultureChange[i], False)
                    self.ChangeNewPlotCulture(i, pCity, CultureChange[i], True)

    def TravellingPerformance(self, pCity):
                iOwner = 0
                Performer = gc.getInfoTypeForString('UNIT_TRAVELLING_PERFORMER')
                for i in range(pCity.plot().getNumUnits()):
                    if pCity.plot().getUnit(i).getUnitType() == Performer:
                        iOwner = pCity.plot().getUnit(i).getOwner()
                        CultureModifier = pCity.getCommerceRateModifier(gc.getInfoTypeForString('COMMERCE_CULTURE')) / 100.0 + 1
                        CultureAmount = int(10 * CultureModifier)
                        pCity.changeCulture(iOwner, CultureAmount, False)
                        self.ChangeNewPlotCulture(iOwner, pCity, CultureAmount)

    def DestroyBuildingBegin(self, argsList):
		pHeadSelectedCity = CyInterface().getHeadSelectedCity()
		popup = PyPopup.PyPopup(7500, EventContextTypes.EVENTCONTEXT_ALL)
		popup.setHeaderString("Choose building to demolish")
		popup.createPullDown()
		AtLeastOneEntry = False
		for i in range( gc.getNumBuildingInfos() ):
			if (i > 0 and pHeadSelectedCity.hasBuilding(i) and not isWorldWonderClass(gc.getBuildingInfo(i).getBuildingClassType())):
				popup.addPullDownString(gc.getBuildingInfo(i).getDescription() , i)
				AtLeastOneEntry = True
#		popup.addPullDownString('Abandon City', i+1)
		popup.addButton("Ok")
		popup.addButton("Cancel")
		if AtLeastOneEntry:
                    popup.launch(False, PopupStates.POPUPSTATE_IMMEDIATE)
		return

    def DestroyBuildingApply(self, playerID, userData, popupReturn):             
		if (popupReturn.getButtonClicked() == 0):
#			if (popupReturn.getSelectedPullDownValue(0) == gc.getNumBuildingInfos() ):
#				pHeadSelectedCity = CyInterface().getHeadSelectedCity()
#				ix = pHeadSelectedCity.getX()
#				iy = pHeadSelectedCity.getY()
#				ipopulation = CyInterface().getHeadSelectedCity().getPopulation()
#				Player = gc.getPlayer(playerID)
#				pHeadSelectedCity.kill()
#				CyAudioGame().Play2DSound("AS2D_DISCOVERBONUS")
#				if (ipopulation > 4):
#					iworkers = ipopulation/5
#					for i in range(0,iworkers):
#						Player.initUnit(5, ix, iy, UnitAITypes.NO_UNITAI)
#			else:
                		iBuildingType = popupReturn.getSelectedPullDownValue(0)
				CyInterface().getHeadSelectedCity().setHasRealBuilding(iBuildingType, False)                 
				CyAudioGame().Play2DSound("AS2D_DISCOVERBONUS")
				CyInterface().setDirty(InterfaceDirtyBits.CityScreen_DIRTY_BIT, True)
				CyInterface().setDirty(InterfaceDirtyBits.SelectionButtons_DIRTY_BIT, True) 
		return

    def UpdateBreweries(self):

#               Get number of players for function
                NumPlayers = 0
		for i in range(gc.getMAX_CIV_PLAYERS()):
			if (gc.getPlayer(i).isAlive()):
				loopPlayer = PyPlayer(gc.getPlayer(i).getID())
				if (loopPlayer.isNone() or not loopPlayer.isAlive()):
					continue
				else:
					NumPlayers += 1

                for i in range(NumPlayers):					
                    for n in range(gc.getPlayer(i).getNumCities()):
                        pCity = gc.getPlayer(i).getCity(n)
                        if pCity.isHasRealBuilding(gc.getInfoTypeForString('BUILDING_OLD_BREWERY')):
                            pCity.setHasRealBuilding(gc.getInfoTypeForString('BUILDING_OLD_BREWERY'), False)
                            pCity.setHasRealBuilding(gc.getInfoTypeForString('BUILDING_NEW_BREWERY'), True)
