## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005

from CvPythonExtensions import *
import CvUtil
import CvEventManager
import PyHelpers
import CvAdvisorUtils
import sys
import CustomFunctions
	
# globals
gc = CyGlobalContext()
PyPlayer = PyHelpers.PyPlayer
cf = CustomFunctions.CustomFunctions()


###################################################
class CvCustomEventManager(CvEventManager.CvEventManager):
	def __init__(self):
		# initialize base class
		self.parent = CvEventManager.CvEventManager
		self.parent.__init__(self)
                self.Events[7500] = ('DestroyBuildingPopup', cf.DestroyBuildingApply , cf.DestroyBuildingBegin)

        def onCityDoTurn(self, argsList):
		'City Production'
		pCity = argsList[0]
		iPlayer = argsList[1]
		
                FoodBonus = cf.CalculateFoodBonus(pCity)
                if not pCity.isFoodProduction():
                    pCity.changeFood(FoodBonus)
                else:
                    pCity.changeProduction(FoodBonus)

                cf.GiveProductionBonus(pCity)
                cf.AssignCustomCulture(pCity, iPlayer)
                cf.TravellingPerformance(pCity)

#               Check this city for bonuses 20 to 34 (Dye, Fur, Gems, Hit Movies, ect)
#                for i in range(15):
#                    if pCity.hasBonus(i+20):
#                        CommerceBonus += 0.05
#                CommerceBonus = int(pCity.getCommerceRate(COMMERCE_GOLD) * CommerceBonus)
#
#                pCity.changeGold(CommerceBonus)

#                if (not pCity.isDisorder()):
#                    CultureBonus = int(pCity.getExtraHappiness() * pCity.getBaseCommerceRate(COMMERCE_CULTURE) * 0.05)
#                    if CultureBonus > 0:
#                        pCity.changeCulture(iPlayer, CultureBonus)

		CvAdvisorUtils.cityAdvise(pCity, iPlayer)
		CvUtil.pyPrint("%s has performed its turn." %(pCity.getName(),))

	def onBuildingBuilt(self, argsList):
		'Building Completed'
		self.parent.onBuildingBuilt(argsList)
		pCity, iBuildingType = argsList

#               If the enhancing buildings are built, assign a free specialist and destroy the enhancing building
                if iBuildingType == gc.getInfoTypeForString('BUILDING_HYDROPONICS_FARM'):
                    pCity.changeFreeSpecialistCount(gc.getInfoTypeForString('SPECIALIST_HYDROPONICS_FARM'), 1)
                    pCity.setHasRealBuilding(iBuildingType, False)
                if iBuildingType == gc.getInfoTypeForString('BUILDING_AUTOMATED_ASSEMBLY_PLANT'):
                    pCity.changeFreeSpecialistCount(gc.getInfoTypeForString('SPECIALIST_AUTOMATED_ASSEMBLY_PLANT'), 1)
                    pCity.setHasRealBuilding(iBuildingType, False)
    		if iBuildingType == gc.getInfoTypeForString('BUILDING_INQUISITION'):
                    cf.Inquisit(pCity)
                    pCity.setHasRealBuilding(iBuildingType, False)
                if iBuildingType == gc.getInfoTypeForString('BUILDING_OLD_BREWERY'):
                    if gc.getPlayer(pCity.getOwner()).isHasTech(gc.getInfoTypeForString('TECH_MEDICINE')):
                        pCity.setHasRealBuilding(gc.getInfoTypeForString('BUILDING_OLD_BREWERY'), False)
                        pCity.setHasRealBuilding(gc.getInfoTypeForString('BUILDING_NEW_BREWERY'), True)

	def onImprovementBuilt(self, argsList):
		'Improvement Built'
		iImprovement, iX, iY = argsList

		pPlot = CyMap().plot(iX,iY)
		if iImprovement == gc.getInfoTypeForString('IMPROVEMENT_TERRAFORM_GRASS'):
			pPlot.setTerrainType(gc.getInfoTypeForString('TERRAIN_IMPROVED_GRASS'), true, true)
			pPlot.setImprovementType(-1)
		if iImprovement == gc.getInfoTypeForString('IMPROVEMENT_TERRAFORM_PLAINS'):
			pPlot.setTerrainType(gc.getInfoTypeForString('TERRAIN_IMPROVED_PLAINS'), true, true)
			pPlot.setImprovementType(-1)
		if iImprovement == gc.getInfoTypeForString('IMPROVEMENT_TERRAFORM_DESERT'):
			pPlot.setTerrainType(gc.getInfoTypeForString('TERRAIN_IMPROVED_DESERT'), true, true)
			pPlot.setImprovementType(-1)
		if iImprovement == gc.getInfoTypeForString('IMPROVEMENT_TERRAFORM_TUNDRA'):
			pPlot.setTerrainType(gc.getInfoTypeForString('TERRAIN_IMPROVED_TUNDRA'), true, true)
			pPlot.setImprovementType(-1)
		if iImprovement == gc.getInfoTypeForString('IMPROVEMENT_TERRAFORM_ICE'):
			pPlot.setTerrainType(CyGlobalContext().getInfoTypeForString('TERRAIN_IMPROVED_ICE'), true, true)
			pPlot.setImprovementType(-1)
		if iImprovement == gc.getInfoTypeForString('IMPROVEMENT_TREE_NURSERY_2'):
			pPlot.setFeatureType(gc.getInfoTypeForString('FEATURE_FOREST'), 0)
			pPlot.setImprovementType(-1)

        def onKbdEvent(self, argsList):
                eventType,Key,mx,my,px,py = argsList
                iHandled = self.parent.onKbdEvent(self, argsList)
#               Destroy building test
#               Check if city screen is active
                if ( iHandled == 0 and CyInterface().isCityScreenUp() ):
#                   Check if CTRL + D was hit
                    if(int(Key) == int(InputTypes.KB_D) and self.bCtrl):
#                      Make sure the owner is the player!
                        if (CyInterface().getHeadSelectedCity().getOwner() == gc.getGame().getActivePlayer()):
#                            cf.DestroyBuildingBegin()
                            self.beginEvent(7500)
                            return 1
                return iHandled

	def onTechAcquired(self, argsList):
		'Tech Acquired'
                self.parent.onTechAcquired(self, argsList)
		iTechType, iTeam, iPlayer, bAnnounce = argsList

		if iTechType == gc.getInfoTypeForString('TECH_MEDICINE'):
                    cf.UpdateBreweries()
